@@ -6,7 +6,7 @@ Redis::JobQueue - Job queue management implemented using Redis server.
 
 =head1 VERSION
 
-This documentation refers to C<Redis::JobQueue> version 1.15
+This documentation refers to C<Redis::JobQueue> version 1.16
 
 =cut
 
@@ -18,7 +18,7 @@ use warnings;
 
 # ENVIRONMENT ------------------------------------------------------------------
 
-our $VERSION = '1.15';
+our $VERSION = '1.16';
 
 use Exporter qw(
     import
@@ -569,6 +569,9 @@ If invoked without any arguments, the constructor C<new> creates and returns
 a C<Redis::JobQueue> object that is configured with the default settings and uses
 local C<redis> server.
 
+In the parameters of the constructor 'redis' may be either a Redis object
+or a server string or a hash reference of parameters to create a Redis object.
+
 Optional C<check_maxmemory> boolean argument (default is true)
 defines if attempt is made to find out maximum available memory from Redis.
 
@@ -608,17 +611,29 @@ This example illustrates C<new()> call with all possible arguments:
                                     # for blocking call of get_next_job.
                                     # Set 0 for unlimited wait time
     );
+    # or
+    my $jq = Redis::JobQueue->new(
+        redis => {                  # The hash reference of parameters
+                                    # to create a Redis object
+            server => "$server:$port",
+        },
+    );
 
 The following examples illustrate other uses of the C<new> method:
 
     $jq = Redis::JobQueue->new();
     my $next_jq = Redis::JobQueue->new( $jq );
 
-    my $redis = Redis->new( redis => "$server:$port" );
+    my $redis = Redis->new( server => "$server:$port" );
     $next_jq = Redis::JobQueue->new(
         $redis,
         timeout => $timeout,
     );
+    # or
+    $next_jq = Redis::JobQueue->new(
+        redis   => $redis,
+        timeout => $timeout,
+    );
 
 An invalid argument causes die (C<confess>).
 
@@ -640,9 +655,12 @@ around BUILDARGS => sub {
     } elsif ( _INSTANCE( $_[0], 'Test::RedisServer' ) ) {
         # to test only
         my $redis = shift;
+        # have to look into the Test::RedisServer object ...
+        my $conf = $redis->conf;
+        $conf->{server} = '127.0.0.1:'.$conf->{port} unless exists $conf->{server};
         return $class->$orig(
-            # have to look into the Test::RedisServer object ...
-            redis   => '127.0.0.1:'.$redis->conf->{port},
+            redis   => $conf->{server},
+            _redis  => Redis->new( %$conf ),
             @_
         );
     } elsif ( _INSTANCE( $_[0], __PACKAGE__ ) ) {
@@ -654,7 +672,32 @@ around BUILDARGS => sub {
             @_
         );
     } else {
-        return $class->$orig( @_ );
+        my %args = @_;
+        my $redis = $args{redis};
+        if ( _INSTANCE( $redis, 'Redis' ) ) {
+            delete $args{redis};
+            return $class->$orig(
+                # have to look into the Redis object ...
+                redis   => $redis->{server},
+                # it is impossible to know from Redis now ...
+                #timeout => $redis->???,
+                _redis  => $redis,
+                %args,
+            );
+        }
+        elsif ( ref( $redis ) eq 'HASH' ) {
+            $args{_use_external_connection} = 0;
+            my $conf = $redis;
+            $conf->{server} = DEFAULT_SERVER.':'.DEFAULT_PORT unless exists $conf->{server};
+            delete $args{redis};
+            return $class->$orig(
+                redis   => $conf->{server},
+                _redis  => Redis->new( %$conf ),
+                %args,
+            );
+        } else {
+            return $class->$orig( %args );
+        }
     }
 };
 
@@ -762,8 +805,7 @@ has '_server'           => (
 
 has '_redis'            => (
     is          => 'rw',
-    # 'Maybe[Test::RedisServer]' to test only
-    isa         => 'Maybe[Redis] | Maybe[Test::RedisServer]',
+    isa         => 'Maybe[Redis]',
     default     => undef,
 );
 
@@ -788,6 +830,12 @@ has '_check_maxmemory'            => (
     default     => 1,
 );
 
+has '_use_external_connection'     => (
+    is          => 'rw',
+    isa         => 'Bool',
+    default     => 1,
+);
+
 #-- public methods -------------------------------------------------------------
 
 =head3 C<last_error>
@@ -1094,7 +1142,7 @@ sub load_job {
     return $new_job;
 }
 
-=head3 C<get_next_job( queue =E<gt> $queue_name, $blocking =E<gt> 1 )>
+=head3 C<get_next_job( queue =E<gt> $queue_name, blocking =E<gt> 1 )>
 
 Selects the job identifier which is at the beginning of the queue.
 
@@ -1188,9 +1236,9 @@ sub get_next_job {
     return;
 }
 
-=head3 C<get_next_job_id( queue =E<gt> $queue_name, $blocking =E<gt> 1 )>
+=head3 C<get_next_job_id( queue =E<gt> $queue_name, blocking =E<gt> 1 )>
 
-Like L</get_nex_job>, but returns job identifier only.
+Like L</get_next_job>, but returns job identifier only.
 
 TTL job data for the job does not reset on the Redis server.
 
@@ -1433,6 +1481,16 @@ The following example illustrates use of the C<ping> method:
 
     $is_alive = $jq->ping;
 
+External connections to the server object (eg, C <$redis = Redis->new( ... );>),
+and the queue object can continue to work after calling ping only if the method returned 1.
+
+If there is no connection to the Redis server (methods return 0), the connection to the server closes.
+In this case, to continue working with the queue,
+you must re-create the C<Redis::JobQueue> object with the L</new> method.
+When using an external connection to the server,
+to check the connection to the server you can use the C<$redis->echo( ... )> call.
+This is useful to avoid closing the connection to the Redis server unintentionally.
+
 =cut
 sub ping {
     my ( $self ) = @_;
@@ -1457,6 +1515,11 @@ The following example illustrates use of the C<quit> method:
 
     $jq->quit;
 
+It does not close the connection to the Redis server if it is an external connection provided
+to queue constructor as existing L<Redis> object.
+When using an external connection (eg, C<$redis = Redis-> new (...);>),
+to close the connection to the Redis server, call C<$redis->quit> after calling this method.
+
 =cut
 sub quit {
     my ( $self ) = @_;
@@ -1465,11 +1528,13 @@ sub quit {
 
     $self->_error( E_NO_ERROR );
 
-    try {
-        $self->_redis->quit;
-    } catch {
-        $self->_redis_exception( $_ );
-    };
+    unless ( $self->_use_external_connection ) {
+        try {
+            $self->_redis->quit;
+        } catch {
+            $self->_redis_exception( $_ );
+        };
+    }
 
     return;
 }
